import josx.platform.rcx.*;

public class LightSensorListener implements SensorListener, Constants {
	
	private static boolean bucketTransport = false;
	
	private static byte round = 1;
	
	public static byte borderBG;

	public static byte borderGW;
	
	private static byte greenMin = 99;

	private static byte greenMax = 0;

	private static Timer greenTimer;

	private static Sensor sensor;
	
	private LineSeek lineSeek;

	private static boolean enabled = false;

	private static boolean turning = false;
	
	public static void init(Sensor sensor) {

		sensor.setTypeAndMode(3, 0x80);
		sensor.activate();

		LightSensorListener.sensor = sensor;

		try {
			initColors();
		} catch (InterruptedException e) {
		}

		greenTimer = new Timer();
		greenTimer.start();
		
	}

	private static void initColors() throws InterruptedException {

		byte colCount = 0;
		
		byte val = 0;
		
		while (colCount < 3) {
			
			LCD.showProgramNumber(colCount);
			
			while(!Button.PRGM.isPressed()) {
				
				LCD.showNumber(sensor.readValue() + 100 * Math.abs(greenMax - greenMin));
				
			}
			
			val = (byte)sensor.readValue();
			
			if (val < greenMin) greenMin = val;
			if (val > greenMax) greenMax = val;
			
			while(Button.PRGM.isPressed());		
			
			colCount++;
			
		}
		
		borderBG = (byte)(greenMin - 1);
		borderGW = (byte)(greenMax + 1);
		
	}

	public LightSensorListener() {

		lineSeek = new LineSeek();
		lineSeek.start();
		
		sensor.addSensorListener(this);

	}

	public static byte getColor(byte val) {

		if (val < borderBG) {

			greenTimer.stopTimer();
			
			return BLACK;
			
		}
		
		if (val < borderGW) {
		
			if (!greenTimer.isActive()) {
				
				if (greenTimer.isExpired())
					return GREEN;
				else 
					greenTimer.startTimer();
				
			}
			
		} else {
			
			greenTimer.stopTimer();
		}
				
		return WHITE;

	}

	public static byte getColor() {

		byte val = (byte) sensor.readValue();

		return getColor(val);

	}

	public synchronized void stateChanged(Sensor source, int oldValue, int newValue) {

		if (!enabled)
			return;
		
		byte color = getColor((byte)newValue);

		
		
		if (color == BLACK) {
			
			turning = false;
			if (lineSeek.isSeeking()) {
				lineSeek.stopSeeking();
			}
			
		}
		else if (color == GREEN) {
			
			if (!turning) {
				turning = true;
				if (lineSeek.isSeeking()) {
					lineSeek.stopSeeking();
				}
				try {
					Thread.sleep(200);
				} catch (InterruptedException e) {
				}
				ConsumerApplication.move(STOP);
				TextLCD.print("turn");
				greenFound();
				LCD.showNumber(round);
				LCD.showProgramNumber(bucketTransport ? 1 : 0);
			}
			
		}
		else { // color == WHITE 
				
			if (!turning && !lineSeek.isSeeking()) {
				lineSeek.startSeeking();
			}
		
		}
		
	}

	public static void greenFound() {
		
		if (bucketTransport) {
			
			ConsumerApplication.move(BACKWARD);
			
			try {
				Thread.sleep(1000);
			} catch (InterruptedException e) {}
			
			round++;
			
			if (round == 4) {
				
				ConsumerApplication.move(STOP);
				Sound.beepSequence();
				AbstractApplication.mainThread.interrupt();
				
			}
			
			ConsumerApplication.move(STOP);
			
			ConsumerApplication.waitForErnie();
			
			ConsumerApplication.move(LEFT);
			
			try {
				Thread.sleep(500);
			} catch (InterruptedException e) {}
			
		} else {
			
			notifyErnie();
			
			ConsumerApplication.move(FORWARD);
			
			try {
				Thread.sleep(500);
			} catch (InterruptedException e) {}
			
			ConsumerApplication.move(RIGHT);
			
			try {
				Thread.sleep(500);
			} catch (InterruptedException e) {}
			
		}
		
		bucketTransport = !bucketTransport;
		
	}
	
	private static void notifyErnie() {
		
		byte[] data = new byte[2];
		
		data[0] = (byte)(data.length - 1); // 2
		data[1] = 23; 
		
		Serial.sendPacket(data,0,data.length);	
		
	}

	public static Sensor getSensor() {
		return sensor;
	}

	public static boolean isEnabled() {
		return enabled;
	}

	public static void setEnabled(boolean aoff) {
		enabled = aoff;
	}

	public static byte getRound() {
		return round;
	}

}
